/*
** Copyright (C) 1998-2010 George Tzanetakis <gtzan@cs.uvic.ca>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef MARSYAS_SPECTRALCENTROIDBANDNORM_H
#define MARSYAS_SPECTRALCENTROIDBANDNORM_H

#include "MarSystem.h"

namespace Marsyas
{
/**
	\class SpectralCentroidBandNorm
	\ingroup Processing
	\brief Basic example on how to use controls efficiently in MarSystems

	This example is the same as Gain; it scales the output by
	multiplying each sample with a real number.

	Controls:
	- \b mrs_real/expected_peak [w] : expected peak in signal
*/

class marsyas_EXPORT SpectralCentroidBandNorm: public MarSystem
{
private:

	/// Add specific controls needed by this MarSystem.
	void addControls();

	/// Reads changed controls and sets up variables if necessary.
	void myUpdate(MarControlPtr sender);

    mrs_real expected_peak_;


public:
	/// SpectralCentroidBandNorm constructor.
	SpectralCentroidBandNorm(std::string name);

	/// SpectralCentroidBandNorm copy constructor.
	SpectralCentroidBandNorm(const SpectralCentroidBandNorm& a);

	/// SpectralCentroidBandNorm destructor.
	~SpectralCentroidBandNorm();

	/// Implementation of the MarSystem::clone() method.
	MarSystem* clone() const;

	/// Implementation of the MarSystem::myProcess method.
	void myProcess(realvec& in, realvec& out);
};

}
//namespace Marsyas

#endif
//MARSYAS_SPECTRALCENTROIDBANDNORM_H

